'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var createReactContext = _interopDefault(require('create-react-context'));
var delve = _interopDefault(require('dlv'));
var unistore = _interopDefault(require('unistore'));
var React = _interopDefault(require('react'));
var hoistNonReactStatics = _interopDefault(require('hoist-non-react-statics'));

var createNamedContext = function createNamedContext(name, defaultValue) {
  var Ctx = createReactContext(defaultValue);
  Ctx.Consumer.displayName = name + ".Consumer";
  Ctx.Provider.displayName = name + ".Provider";
  return Ctx;
};

var _createNamedContext = createNamedContext('RouterHook'),
    Provider = _createNamedContext.Provider,
    Consumer = _createNamedContext.Consumer;

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

var propName = '@@react-router-hook';

var actions = {
  startLoading: function startLoading(state, key, version) {
    var _objectSpread2;

    if (state[key] && state[key].version === version && !!state[key].loading) {
      return state;
    }

    return _objectSpread({}, state, (_objectSpread2 = {}, _objectSpread2[key] = _objectSpread({}, state[key] || {}, {
      version: version,
      loading: true
    }), _objectSpread2));
  },
  startTriggerHook: function startTriggerHook(state, key, hook, version) {
    var _objectSpread3, _objectSpread4;

    if (!state[key] || state[key].version !== version || delve(state, [key, 'hooks', hook]) === version) {
      return state;
    }

    return _objectSpread({}, state, (_objectSpread4 = {}, _objectSpread4[key] = _objectSpread({}, state[key], {
      hooks: _objectSpread({}, delve(state, [key, 'hooks'], {}), (_objectSpread3 = {}, _objectSpread3[hook] = version, _objectSpread3))
    }), _objectSpread4));
  },
  finishLoading: function finishLoading(state, key, version) {
    var _objectSpread5;

    if (!state[key] || state[key].version !== version) {
      return state;
    }

    return _objectSpread({}, state, (_objectSpread5 = {}, _objectSpread5[key] = _objectSpread({}, state[key], {
      loading: false
    }), _objectSpread5));
  },
  updateProps: function updateProps(state, key, props, version) {
    var _objectSpread6;

    if (!state[key] || state[key].version !== version) {
      return state;
    }

    return _objectSpread({}, state, (_objectSpread6 = {}, _objectSpread6[key] = _objectSpread({}, state[key], {
      props: props
    }), _objectSpread6));
  }
};

function mapActions(actionMap, store) {
  var acts = typeof actionMap === 'function' ? actionMap(store) : actionMap;
  var mapped = {};
  Object.keys(acts).forEach(function (k) {
    mapped[k] = store.action(acts[k]);
  });
  return mapped;
}

function noop () {}

function unwrap(Component, locals) {
  return typeof locals === 'function' ? unwrap(Component, locals(Component)) : locals;
}

function triggerHooks(component, triggerConfig, version, force) {
  var components = (Array.isArray(component) ? component : [component]).filter(function (c) {
    return c && c[propName];
  });

  if (components.length < 1) {
    return Promise.resolve();
  }

  var hookNames = triggerConfig.hookNames,
      locals = triggerConfig.locals,
      store = triggerConfig.store,
      onError = triggerConfig.onError;
  var boundActions = mapActions(actions, store);

  var getLocals = function getLocals(Component) {
    var componentHooks = Component[propName];
    return _objectSpread({}, unwrap(Component, locals) || {}, {
      setProps: componentHooks ? function (props) {
        return boundActions.updateProps(componentHooks.id, props, version);
      } : noop,
      getProps: componentHooks ? function () {
        return delve(store.getState(), [componentHooks.id, 'props']);
      } : noop
    });
  };

  var promises = components.map(function (Component) {
    var componentHooks = Component[propName];
    if (!componentHooks) return null;
    var hookId = componentHooks.id;
    var runHookNames = hookNames.filter(function (key) {
      return componentHooks[key];
    });
    if (runHookNames.length < 1) return null;
    boundActions.startLoading(hookId, version);
    return runHookNames.reduce(function (total, name) {
      return total.then(function () {
        if (!force && delve(store.getState(), [hookId, 'hooks', name]) === version) return null;
        boundActions.startTriggerHook(hookId, name, version);
        return componentHooks[name](getLocals(Component));
      });
    }, Promise.resolve()).then(function () {
      boundActions.finishLoading(hookId, version);
    }).catch(function (err) {
      boundActions.finishLoading(hookId, version);

      if (onError) {
        onError({
          Component: Component,
          error: err
        });
        return;
      }

      throw err;
    });
  }).filter(function (p) {
    return p;
  });
  return Promise.all(promises);
}

var DEFAULT_HOOK_OPTIONS = {
  withRef: false,
  blockMode: true,
  exposeLoading: false,
  exposeReloadComponent: false
};
var contextProp = '@@hook_ctx';
var forwardedRef = '@@hook_ref';

function getDisplayName(Component) {
  return Component.displayName || Component.name || (typeof Component === 'string' && Component.length > 0 ? Component : 'Unknown');
}

function withForwardRef(withRef) {
  return withRef && React.forwardRef ? function (fn) {
    return React.forwardRef(function (props, ref) {
      return fn(props, ref);
    });
  } : function (fn) {
    return function (props) {
      return fn(props);
    };
  };
}

var routerHooks = function routerHooks(hooks, hookOpts) {
  var hookOptions = _objectSpread({}, DEFAULT_HOOK_OPTIONS, hookOpts || {});

  return function (Component) {
    var componentDisplayName = getDisplayName(Component);
    /* eslint no-param-reassign:0 */

    hooks.id = hooks.id || componentDisplayName + "_" + Math.random().toString(36).substr(2, 9);
    Component[propName] = hooks;
    /* eslint no-param-reassign:1 */

    var RouterHookLoadable =
    /*#__PURE__*/
    function (_React$Component) {
      _inheritsLoose(RouterHookLoadable, _React$Component);

      function RouterHookLoadable(props) {
        var _this;

        _this = _React$Component.call(this, props) || this;
        _this.updateState = _this.updateState.bind(_assertThisInitialized(_assertThisInitialized(_this)));
        _this.reloadComponent = _this.reloadComponent.bind(_assertThisInitialized(_assertThisInitialized(_this)));
        _this.state = {
          loading: false,
          props: null
        };
        return _this;
      }

      var _proto = RouterHookLoadable.prototype;

      _proto.componentDidMount = function componentDidMount() {
        var context = this.props[contextProp];
        var triggerConfig = context.triggerConfig;
        var store = triggerConfig.store;
        this.unsubscribe = store.subscribe(this.updateState);
        setTimeout(this.reloadComponent);
      };

      _proto.shouldComponentUpdate = function shouldComponentUpdate(nextProps, nextState) {
        var context = this.props[contextProp];
        var nextContext = nextProps[contextProp];
        var loading = this.state.loading;
        if (!hookOptions.blockMode) return true;

        if (hookOptions.exposeLoading && loading !== nextState.loading) {
          return true;
        }

        return !nextState.loading || context.version !== nextContext.version;
      };

      _proto.componentDidUpdate = function componentDidUpdate(prevProps) {
        var context = this.props[contextProp];
        var prevContext = prevProps[contextProp];

        if (context.version !== prevContext.version) {
          setTimeout(this.reloadComponent);
        }
      };

      _proto.componentWillUnmount = function componentWillUnmount() {
        if (this.unsubscribe) {
          this.unsubscribe();
          this.unsubscribe = undefined;
        }
      };

      _proto.updateState = function updateState() {
        var context = this.props[contextProp];
        var triggerConfig = context.triggerConfig;
        var store = triggerConfig.store;
        var loading = delve(store.getState(), [hooks.id, 'loading'], false);
        var props = delve(store.getState(), [hooks.id, 'props'], null);
        var _this$state = this.state,
            currentLoading = _this$state.loading,
            currentProps = _this$state.props;

        if (loading !== currentLoading || props !== currentProps) {
          this.setState({
            loading: loading,
            props: props
          });
        }
      };

      _proto.reloadComponent = function reloadComponent() {
        var context = this.props[contextProp];
        var triggerConfig = context.triggerConfig,
            version = context.version;
        return triggerHooks(Component, triggerConfig, version);
      };

      _proto.render = function render() {
        var _this$props = this.props,
            context = _this$props[contextProp],
            ref = _this$props[forwardedRef],
            restProps = _objectWithoutPropertiesLoose(_this$props, [contextProp, forwardedRef]);

        var triggerConfig = context.triggerConfig;
        var store = triggerConfig.store;
        var loading = this.state.loading;

        var props = _objectSpread({}, restProps, delve(store.getState(), [hooks.id, 'props'], {}));

        if (hookOptions.exposeLoading) props.loading = loading;
        if (hookOptions.exposeReloadComponent) props.reloadComponent = this.reloadComponent;
        if (ref) props.ref = ref;
        return React.createElement(Component, props);
      };

      return RouterHookLoadable;
    }(React.Component);

    RouterHookLoadable.displayName = "RouterHookLoadable(" + componentDisplayName + ")";
    var WithRouterHookConsumer = withForwardRef(hookOptions.withRef)(function (props, ref) {
      return React.createElement(Consumer, null, function (context) {
        var _objectSpread2;

        var passProps = _objectSpread({}, props, (_objectSpread2 = {}, _objectSpread2[contextProp] = context, _objectSpread2));

        if (ref) passProps[forwardedRef] = ref;
        return React.createElement(RouterHookLoadable, passProps);
      });
    });
    WithRouterHookConsumer.WrappedComponent = Component;
    hoistNonReactStatics(WithRouterHookConsumer, Component);
    WithRouterHookConsumer.displayName = "@routerHooks(" + componentDisplayName + ")";
    return WithRouterHookConsumer;
  };
};

exports.createHookStore = unistore;
exports.routerHookPropName = propName;
exports.RouterHookProvider = Provider;
exports.RouterHookConsumer = Consumer;
exports.routerHooks = routerHooks;
exports.triggerHooks = triggerHooks;
